@@ -36,7 +36,7 @@
 #define INT24_MAX 0x007fffffL
 #endif
 
-/* Thanks you
+/* Thank you
  * http://www.linux-nantes.org/~fmonnier/ocaml/ocaml-wrapping-c.php#ref_option */
 value flac_Val_some( value v )
 {   
@@ -123,22 +123,7 @@ CAMLprim value caml_flac_s16le_to_float(value _src, value _chans)
 /* Decoder */
 
 /* polymorphic variant utility macros */
-#define decl_var(x) static value var_##x
-#define import_var(x) var_##x = caml_hash_variant(#x)
-#define get_var(x) var_##x
-
-/* cached polymorphic variants */
-decl_var(Search_for_metadata);
-decl_var(Read_metadata);
-decl_var(Search_for_frame_sync);
-decl_var(Read_frame);
-decl_var(End_of_stream);
-decl_var(Ogg_error);
-decl_var(Seek_error);
-decl_var(Aborted);
-decl_var(Memory_allocation_error);
-decl_var(Uninitialized);
-decl_var(Unknown);
+#define get_var(x) caml_hash_variant(#x)
 
 static value val_of_state(int s) {
   switch (s)
@@ -184,25 +169,6 @@ static value raise_exn_of_error(FLAC__StreamDecoderErrorStatus e) {
     }
 }
 
-/* initialize the module */
-CAMLprim value ocaml_flac_stubs_initialize(value unit)
-{
-  CAMLparam0();
-  /* initialize polymorphic variants */
-  import_var(Search_for_metadata);
-  import_var(Read_metadata);
-  import_var(Search_for_frame_sync);
-  import_var(Read_frame);
-  import_var(End_of_stream);
-  import_var(Ogg_error);
-  import_var(Seek_error);
-  import_var(Aborted);
-  import_var(Memory_allocation_error);
-  import_var(Uninitialized);
-  import_var(Unknown);
-  CAMLreturn(Val_unit);
-}
-
 /* Caml abstract value containing the decoder. */
 #define Decoder_val(v) (*((ocaml_flac_decoder**)Data_custom_val(v)))
 
@@ -549,7 +515,10 @@ CAMLprim value ocaml_flac_decoder_info(value d)
      m = caml_alloc_tuple(2);
      FLAC__StreamMetadata_VorbisComment coms = dec->callbacks.meta->data.vorbis_comment;
      // First comment is vendor string
-     Store_field(m,0,caml_copy_string((char *)coms.vendor_string.entry));
+     if (coms.vendor_string.entry != NULL)
+       Store_field(m,0,caml_copy_string((char *)coms.vendor_string.entry));
+     else
+       Store_field(m,0,caml_copy_string(""));
      // Now the other metadata
      tmp = caml_alloc_tuple(coms.num_comments);
      int i;
@@ -769,7 +738,7 @@ value ocaml_flac_encoder_alloc(value comments, value params, struct custom_opera
   caml_enc->callbacks.private = NULL;
   caml_register_global_root(&caml_enc->callbacks.callbacks);
   caml_enc->callbacks.callbacks = Val_none;
-  caml_register_global_root(&caml_enc->callbacks.callbacks);
+  caml_register_global_root(&caml_enc->callbacks.tmp);
   caml_enc->callbacks.tmp = Val_none;
   caml_enc->buf = NULL;
   caml_enc->lines = NULL;
@@ -818,7 +787,7 @@ CAMLprim value ocaml_flac_encoder_create(value comments, value params, value cal
                                    enc_tell_callback,
                                    NULL,
                                    (void*)&enc->callbacks);
-  caml_enter_blocking_section();
+  caml_leave_blocking_section();
 
   Free_values(enc);
 
